package com.winhc.max.compute.graph.job.holder;

import com.aliyun.odps.data.TableInfo;
import com.aliyun.odps.graph.GraphJob;
import com.aliyun.odps.graph.RemoveDuplicatesLoadingResolver;
import com.winhc.max.compute.graph.job.pagerank.CompanyComputingVertexResolver;
import com.winhc.max.compute.graph.util.ParameterTool;

import java.io.IOException;

/**
 * @author π
 * @Description:个人关系聚合
 * @date 2024/5/28 16:25
 */
public class HolderRelationGroupJob {


    private static final String defaultInputVertexTab;
    private static final String defaultOutputTab;

    static {
        defaultInputVertexTab = "tmp_xf_holder_relation_in";
        defaultOutputTab = "tmp_xf_holder_relation_out";
    }


    public static void main(String[] args) throws IOException {

        ParameterTool parameterTool = ParameterTool.fromArgs(args);

        String numWorkers = parameterTool.getOrDefault("numWorkers", "70");
        String workerCPU = parameterTool.getOrDefault("workerCPU", "2");
        String workerMem = parameterTool.getOrDefault("workerMem", "8192");

        String inputVertexTab = parameterTool.getOrDefault("input", defaultInputVertexTab);
        String outputParentTab = parameterTool.getOrDefault("output", defaultOutputTab);
        String run_step = parameterTool.getOrDefault("run_step", "1");
        String node_limit = parameterTool.getOrDefault("node_limit", "100");
        String jvm_ratio = parameterTool.getOrDefault("jvm_ratio", "0.7");

        System.out.println("input args: " + String.join(" ", args));
        GraphJob job = new GraphJob();

        job.setNumWorkers(Integer.parseInt(numWorkers));
        if (parameterTool.has("workerCPU")) {
            job.setWorkerCPU(Integer.parseInt(workerCPU) * 100);
        }
        if (parameterTool.has("workerMem")) {
            job.setWorkerMemory(Integer.parseInt(workerMem));
        }

        job.setGraphLoaderClass(HolderRelationGroupReader.class);
        job.setVertexClass(HolderRelationGroupVertex.class);

        job.setLoadingVertexResolver(RemoveDuplicatesLoadingResolver.class);
        job.setComputingVertexResolver(CompanyComputingVertexResolver.class);
        job.setAggregatorClass(HolderRelationGroupAggregator.class);

        job.addInput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName(inputVertexTab)
                .build());

        job.addOutput(TableInfo.builder()
                .projectName("winhc_ng")
                .tableName(outputParentTab)
                .build());

        job.set("run_step", run_step);
        job.set("node_limit", node_limit);
        job.set("odps.graph.jvm.xms.ratio", jvm_ratio);

        long startTime = System.currentTimeMillis();
        job.run();
        System.out.println("Job Finished in "
                + (System.currentTimeMillis() - startTime) / 1000.0 + " seconds");


    }
}
