package com.winhc.max.compute.graph.job.holder;

import com.aliyun.odps.conf.Configuration;
import com.aliyun.odps.data.TableInfo;
import com.aliyun.odps.graph.Edge;
import com.aliyun.odps.graph.GraphLoader;
import com.aliyun.odps.graph.MutationContext;
import com.aliyun.odps.io.LongWritable;
import com.aliyun.odps.io.Text;
import com.aliyun.odps.io.WritableRecord;
import com.aliyun.odps.utils.StringUtils;
import com.winhc.max.compute.graph.job.holder.entity.HolderRelationGroupEdge;
import com.winhc.max.compute.graph.job.holder.entity.HolderRelationGroupMsg;
import com.winhc.max.compute.graph.job.holder.entity.HolderRelationGroupVertexValue;
import com.winhc.max.compute.graph.util.BaseUtils;
import com.winhc.max.compute.graph.util.JsonUtils;
import com.winhc.max.compute.graph.util.WritableRecordExtensions;
import lombok.experimental.ExtensionMethod;

import java.io.IOException;
import java.util.Arrays;

/**
 * @author π
 * @Description:
 * @date 2024/5/29 9:28
 */

@ExtensionMethod({
        JsonUtils.class
        , WritableRecordExtensions.class
        , BaseUtils.class
})
public class HolderRelationGroupReader extends
        GraphLoader<Text, HolderRelationGroupVertexValue, HolderRelationGroupEdge, HolderRelationGroupMsg> {


    @Override
    public void setup(Configuration conf, int workerId, TableInfo tableInfo, MutationContext<Text, HolderRelationGroupVertexValue, HolderRelationGroupEdge, HolderRelationGroupMsg> context) throws IOException {
        super.setup(conf, workerId, tableInfo, context);
    }

    @Override
    public void load(LongWritable recordNum, WritableRecord record, MutationContext<Text, HolderRelationGroupVertexValue, HolderRelationGroupEdge, HolderRelationGroupMsg> context) throws IOException {


        Text companyId = record.getTextOrNull("company_id");
        Text companyName = record.getTextOrNull("company_name");


        Text holderId = record.getTextOrNull("holder_id");
        Text holderName = record.getTextOrNull("holder_name");

        //加载点起始点
        HolderRelationGroupVertex holderVertex = new HolderRelationGroupVertex();
        holderVertex.setId(companyId);
        holderVertex.setValue(HolderRelationGroupVertexValue.of(
                companyName
                /*, Arrays.asList(companyId.toString(), holderId.toString())*/
                /*, Collections.singletonList(
                        String.join(",", companyId.toString(), holderId.toString()))*/
                , Arrays.asList(holderId.toString())
        ));
        context.addVertexRequest(holderVertex);


//        //加载目标点
//        HolderRelationGroupVertex holderVertexDest = new HolderRelationGroupVertex();
//        holderVertexDest.setId(holderId);
//        holderVertexDest.setValue(HolderRelationGroupVertexValue.of(
//                holderName
//                , Arrays.asList(companyId.toString())
//        ));
//        context.addVertexRequest(holderVertexDest);

        //加载边
        if(StringUtils.isNotBlank(holderId.toString())){
            HolderRelationGroupEdge edgeValue = HolderRelationGroupEdge.of(holderId.toString(), 1.0);
            Edge<Text, HolderRelationGroupEdge> edge = new Edge<Text, HolderRelationGroupEdge>(
                    holderId, edgeValue);
            context.addEdgeRequest(companyId, edge);
        }


    }
}