package com.winhc.max.compute.graph.job.holder.entity;

import com.aliyun.odps.io.*;
import com.winhc.max.compute.graph.job.enterprise_group.EnterpriseGroupUtils;
import com.winhc.max.compute.graph.job.enterprise_group.EnterpriseGroupVertex;
import lombok.experimental.ExtensionMethod;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import static com.winhc.max.compute.graph.job.holder.HolderRelationGroupVertex.list2Tuple2;


public class HolderRelationGroupMsg implements Writable {

//    private Tuple holderChains;

//    private Tuple companyIds;

    /**
     * 外部扩展id
     */
    private Tuple extendChains;

    /**
     * 途径id记录，防止陷入循环
     */
    private Tuple routeLog;

    /**
     * 发送消息id
     */
    private Text sendVertexId;


    /**
     * 接收消息id
     */
    private Text receiveVertexId;


    /**
     * 记录并测试是否重复
     *
     * @param vertexId
     * @return false表示该节点已经走过。
     */
    public boolean routeLog(Text vertexId) {
        return routeLog(vertexId.toString());
    }

    public boolean routeLog(String vertexId) {
        long count = routeLog.getAll().stream().map(Object::toString).filter(e -> e.equals(vertexId)).count();

        if (count > 0) {
//            System.out.println("陷入循环，" + String.join(",", routeLog.tuple2List()));
            return false;
        }
        routeLog.append(new Text(vertexId));
        return true;

    }

    public static HolderRelationGroupMsg of(String sendVertexId, String receiveVertexId, List<Set<String>> extendChains,
                                            HashSet<String> companyIds) {
        HolderRelationGroupMsg holderRelationGroupMsg = new HolderRelationGroupMsg();
        //holderRelationGroupMsg.companyIds = new Tuple();

//        for (String id : companyIds) {
//            holderRelationGroupMsg.companyIds.append(new Text(id));
//        }



        holderRelationGroupMsg.sendVertexId = new Text(sendVertexId);
        holderRelationGroupMsg.receiveVertexId = new Text(receiveVertexId);

        //扩展集合
        holderRelationGroupMsg.extendChains = new Tuple();
        for (Set<String> ex : extendChains) {
            Tuple tuple2 = new Tuple();
            for (String e : ex) {
                tuple2.append(new Text(e));
            }
            holderRelationGroupMsg.extendChains.append(tuple2);
        }

        return holderRelationGroupMsg;
    }


    @Override
    public void write(DataOutput out) throws IOException {
        //companyIds.write(out);

        //holderChains.write(out);
        extendChains.write(out);

        sendVertexId.write(out);
        receiveVertexId.write(out);
    }

    @Override
    public void readFields(DataInput in) throws IOException {
        //companyIds = new Tuple();

        //holderChains = new Tuple();
        extendChains = new Tuple();

        //companyIds.readFields(in);
        //holderChains.readFields(in);
        extendChains.readFields(in);


        sendVertexId = new Text();
        receiveVertexId = new Text();

        sendVertexId.readFields(in);
        receiveVertexId.readFields(in);
    }

//    public Tuple getHolderChains() {
//        return holderChains;
//    }
//
//    public void setHolderChains(Tuple holderChains) {
//        this.holderChains = holderChains;
//    }

//    public Tuple getCompanyIds() {
//
//        return companyIds;
//    }

    public Tuple getCompanyIds() {
        if (extendChains == null) {
            return new Tuple();
        }
        List<String> collect = extendChains.getAll().stream()
                .filter(Objects::nonNull).flatMap(xx->{
                    return  ((Tuple)xx).getAll().stream().filter(Objects::nonNull).map(e -> ((Text) e).toString());
                })
                .distinct().collect(Collectors.toList());
        Tuple res = list2Tuple2(collect);

        return res;
    }

//    public void setCompanyIds(Tuple companyIds) {
//        this.companyIds = companyIds;
//    }

    public Tuple getRouteLog() {
        return routeLog;
    }

    public void setRouteLog(Tuple routeLog) {
        this.routeLog = routeLog;
    }

    public Text getSendVertexId() {
        return sendVertexId;
    }

    public void setSendVertexId(Text sendVertexId) {
        this.sendVertexId = sendVertexId;
    }

    public Text getReceiveVertexId() {
        return receiveVertexId;
    }

    public void setReceiveVertexId(Text receiveVertexId) {
        this.receiveVertexId = receiveVertexId;
    }

    public Tuple getExtendChains() {
        return extendChains;
    }

    public void setExtendChains(Tuple extendChains) {
        this.extendChains = extendChains;
    }
}
